@@ -28,7 +28,7 @@ def run_init_config(args):
 
     init_airflow_db(args)
     patch_airflow_config(args.config)
-    add_connections(args)
+    # add_connections(args)
     if args.upgrade:
         upgrade_dags(args.config)
     copy_dags(args.home)
@@ -70,6 +70,7 @@ def patch_airflow_config(airflow_config):
     patch = f"""
     sed -i -e 's/^dags_are_paused_at_creation.*/dags_are_paused_at_creation = False/g' {airflow_config} && \
     sed -i -e 's/^load_examples.*/load_examples = False/g' {airflow_config} && \
+    sed -i -e 's/^logging_config_class.*/logging_config_class = cwl_airflow.config_templates.airflow_local_settings.DEFAULT_LOGGING_CONFIG/g' {airflow_config} && \
     sed -i -e 's/^hide_paused_dags_by_default.*/hide_paused_dags_by_default = True/g' {airflow_config}
     """
 
@@ -138,7 +139,7 @@ def copy_dags(airflow_home, source_folder=None):
                 if not os.path.isfile(os.path.join(target_folder, filename)):
                     shutil.copy(os.path.join(root, filename), target_folder)
 
-
+# not used anymore
 def add_connections(args):
     """
     Sets AIRFLOW_HOME and AIRFLOW_CONFIG from args.
