@@ -36,7 +36,7 @@ def get_listener_thread(                                       # safe to kill wh
     port,
     daemon
 ):
-    httpd = socketserver.TCPServer(("", port), CustomHandler)
+    httpd = socketserver.TCPServer(("127.0.0.1", port), CustomHandler)
     httpd.results_queue = results_queue                        # to have access to results_queue from CustomHandler through self.server.results_queue
     return threading.Thread(
         target=httpd.serve_forever,
@@ -63,6 +63,9 @@ def get_checker_thread(                                        # is not thread s
 
 class CustomHandler(SimpleHTTPRequestHandler):
 
+    def log_message(self, format, *args):  # to suppress logging on each POST 200 response sent
+        pass
+
     def do_POST(self):
         self.send_response(200)
         self.end_headers()
@@ -73,7 +76,7 @@ class CustomHandler(SimpleHTTPRequestHandler):
                 int(self.headers["Content-Length"])
             ).decode("UTF-8")
         )["payload"]
-        if payload.get("results", None) or payload.get("state", None) == "failed":
+        if "results" in payload or payload.get("state", None) == "failed":     # "results" can be {}, so we should check only if key is present, but not value
             self.server.results_queue.put({
                 "run_id":  payload["run_id"],
                 "dag_id":  payload["dag_id"],
@@ -105,22 +108,30 @@ def get_unfinished_runs(suite_data):
 
 
 def check_result(suite_data, results_queue):
+    previous_unfinished_count = None
     while True:
         suite_data_unfinished = get_unfinished_runs(suite_data)
         if len(suite_data_unfinished) == 0:
             break
         try:
-            item = results_queue.get()
+            item = results_queue.get(False)
         except queue.Empty:
-            sleep(3)                               # sleep for 3 second before trying to fetch new results from the empty queue
+            unfinished_count = len(suite_data_unfinished)
+            if previous_unfinished_count != unfinished_count:
+                logging.info(f"Waiting for {unfinished_count} unfinished runs:")
+                for run_id, test_data in suite_data_unfinished.items():
+                    logging.info(f"   test case - {test_data['index']}, dag_id - {test_data['dag_id']}, run_id - {run_id}")   
+                previous_unfinished_count = unfinished_count
+            sleep(10)                                       # sleep for 10 second before trying to fetch new results from the empty queue
             continue
         run_id = item["run_id"]
         test_data = suite_data_unfinished[run_id]  # if this fails, look for a bug
         logging.info(f"Check results from the test case {test_data['index']} that runs DAG {test_data['dag_id']} as {run_id}")
         try:
             compare(test_data["output"], item["results"])
-        except CompareFail as ex:
-            test_data["error"] = str(ex)
+        except (CompareFail, KeyError) as ex:               # catch KeyError in case output field is missing for tool that should fail
+            if not test_data.get("should_fail", None):      # do not report error if tool should fail
+                test_data["error"] = str(ex)
         finally:
             test_data["finished"] = True
             rmtree(test_data["job"]["outputs_folder"])
@@ -148,14 +159,18 @@ def load_test_suite(args):
         test_data = suite_data[i]
         run_id = str(uuid.uuid4())
         tool_location = get_absolute_path(test_data["tool"], suite_dir)
-        job_location = get_absolute_path(test_data["job"], suite_dir)
+        logging.info(f"Read test case {i+1} to run {tool_location}")
+
+        job_location = None
+        job_data = {}
+
         if "job" in test_data:
+            job_location = get_absolute_path(test_data["job"], suite_dir)
             job_data = load_job(
                 workflow=tool_location,
                 job=job_location
             )
-        else:
-            job_data = {}
+
         job_data["outputs_folder"] = get_dir(os.path.join(args.tmp, run_id))
 
         test_data.update({
@@ -165,7 +180,7 @@ def load_test_suite(args):
             "index": i+1,                                                     # to know test case number, 1-based to correspond to --range
             "finished": False                                                 # to indicate whether the test was finished or not
         })
-        logging.info(f"Load test case {i+1} to run {tool_location} with {job_location} as {run_id}")
+        logging.info(f"Successfully loaded test case {i+1} to run {tool_location} with {job_location} as {run_id}")
         suite_data_filtered[run_id] = test_data                               # use "run_id" as a key for fast access when checking results
     return suite_data_filtered
 
@@ -193,8 +208,7 @@ def create_dags(suite_data, args, dags_folder=None):
             location=workflow_path
         )
         with open(workflow_path, "rb") as input_stream:
-            logging.info(f"Add DAG {test_data['dag_id']} \
-                from test case {test_data['index']}")
+            logging.info(f"Add DAG {test_data['dag_id']} from test case {test_data['index']}")
 
             if args.embed:                                                                            # send base64 encoded zlib compressed workflow content that will be embedded into DAG python file
                 logging.info(f"Sending base64 encoded zlib compressed content from {workflow_path}")
@@ -221,8 +235,7 @@ def create_dags(suite_data, args, dags_folder=None):
 
             if not r.ok:
                 reason = get_api_failure_reason(r)
-                logging.error(f"Failed to add DAG {test_data['dag_id']} \
-                    from test case {test_data['index']} due to {reason}")
+                logging.error(f"Failed to add DAG {test_data['dag_id']} from test case {test_data['index']} due to \n {reason}")
 
 
 def trigger_dags(suite_data, args):
