@@ -5,13 +5,11 @@ import dill as pickle  # standard pickle doesn't handle lambdas
 import argparse
 import json
 import zlib
-import errno
 import shutil
 import docker
 import logging
 import binascii
 
-from uuid import uuid4
 from copy import deepcopy
 from jsonmerge import merge
 from urllib.parse import urlsplit
@@ -47,7 +45,6 @@ from schema_salad.ref_resolver import file_uri
 from cwl_airflow.utilities.helpers import (
     get_md5_sum,
     get_dir,
-    get_path_from_url,
     load_yaml,
     dump_json,
     get_absolute_path,
@@ -61,6 +58,7 @@ from cwl_airflow.utilities.helpers import (
 
 CWL_TMP_FOLDER = os.path.join(AIRFLOW_HOME, "cwl_tmp_folder")
 CWL_OUTPUTS_FOLDER = os.path.join(AIRFLOW_HOME, "cwl_outputs_folder")
+CWL_INPUTS_FOLDER = os.path.join(AIRFLOW_HOME, "cwl_inputs_folder")
 CWL_PICKLE_FOLDER = os.path.join(AIRFLOW_HOME, "cwl_pickle_folder")
 CWL_USE_CONTAINER = True
 CWL_NO_MATCH_USER = False
@@ -152,13 +150,17 @@ def conf_get(
 ):
     """
     Return value from AirflowConfigParser object.
-    If section or key is absent, return default
+    If section or key is absent, return default.
+    Suppresses annoying warning messages
     """
 
     try:
+        logging.disable(logging.WARNING)
         return conf.get(section, key)
     except AirflowConfigException:
         return default
+    finally:
+        logging.disable(logging.NOTSET)  # is guaranteed to be executed before any return
 
 
 def collect_reports(
@@ -218,6 +220,12 @@ def get_default_cwl_args(preset_cwl_args=None):
                     preset_cwl_args.get("outputs_folder", CWL_OUTPUTS_FOLDER)
                 )
             ),
+            "inputs_folder": get_dir(                                             # for CWL-Airflow to resolve relative locations for input files if job was loaded from parsed object
+                conf_get(
+                    "cwl", "inputs_folder",
+                    preset_cwl_args.get("inputs_folder", CWL_INPUTS_FOLDER)
+                )
+            ),
             "pickle_folder": get_dir(                                              # for CWL-Airflow to store pickled workflows
                 conf_get(
                     "cwl", "pickle_folder",
@@ -416,7 +424,7 @@ def execute_workflow_step(
         location=workflow_step_path
     )
 
-    _stderr = sys.stderr                               # to trick the logger
+    _stderr = sys.stderr                               # to trick the logger	
     sys.stderr = sys.__stderr__
     step_outputs, step_status = executor(
         slow_cwl_load(
@@ -487,13 +495,15 @@ def load_job(
     inputs; never fails). "cwl_args" can be used to update parameters for
     loading and runtime contexts.
 
-    If "job" was file, resolves relative paths based on the job file
-    location. If "job" was already parsed Object and "cwd" is not None,
-    then relative paths will be resolved based on "cwd". Otherwise, relative
-    paths will not be resolved.
+    If "job" was file, resolves relative paths based on the job file location.
+    If "job" was already parsed into Object, resolves relative paths based on
+    "cwd". If "cwd" was None uses "inputs_folder" value from "cwl_args" or
+    its default value returned from "get_default_cwl_args" function.
 
-    Checks links only when relative paths are resolved, failing on
-    missing input files.
+    Checking links after relative paths are resolved is disabled (checklinks
+    is set to False in both places). This will prevent rasing an exception by
+    schema salad in those cases when an input file will be created from the
+    provided content during workflow execution.
     
     Always returns CommentedMap
     """
@@ -501,6 +511,7 @@ def load_job(
     cwl_args = {} if cwl_args is None else cwl_args
     
     default_cwl_args = get_default_cwl_args(cwl_args)
+    cwd = default_cwl_args["inputs_folder"] if cwd is None else cwd
 
     loading_context = setup_loadingContext(
         LoadingContext(default_cwl_args),
@@ -511,16 +522,15 @@ def load_job(
     job_copy = deepcopy(job)
 
     try:
-        job_data, _ = loading_context.loader.resolve_ref(job_copy, checklinks=True)
+        job_data, _ = loading_context.loader.resolve_ref(job_copy, checklinks=False)
     except (FileNotFoundError, SchemaSaladException) as err:
         job_data = load_yaml(json.dumps(job_copy))
-        if cwd is not None:
-            job_data["id"] = file_uri(cwd) + "/"
-            job_data, metadata = loading_context.loader.resolve_all(
-                job_data,
-                job_data["id"],
-                checklinks=True
-            )
+        job_data["id"] = file_uri(cwd) + "/"
+        job_data, metadata = loading_context.loader.resolve_all(
+            job_data,
+            job_data["id"],
+            checklinks=False
+        )
 
     initialized_job_data = init_job_order(
         job_order_object=job_data,
@@ -542,6 +552,8 @@ def fast_cwl_step_load(workflow, target_id, cwl_args=None, location=None):
     selected by "target_id" from the parsed "workflow". Other steps
     are removed. Workflow inputs and outputs are updated based on
     source fields of "in" and "out" from the selected workflow step.
+    If selected step includes "scatter" field all output types will
+    be transformed to the nested/flat array of items of the same type. 
     IDs of updated workflow inputs and outputs as well as IDs of
     correspondent "source" fields also include step id separated by
     underscore. All other fields remain unchanged.
@@ -590,8 +602,12 @@ def fast_cwl_step_load(workflow, target_id, cwl_args=None, location=None):
                     "type": workflow_input["type"]
                 }
 
-                # need to copy both "default" and "secondaryFiles" if present
-                for key in ["default", "secondaryFiles"]:  # TODO: Do I need to copy format?
+                # need to copy:
+                #  original inputBinding because it can include loadContents section
+                #  loadContents itself if present outside of inputBinding
+                #  both "default" and "secondaryFiles" if present
+                # TODO: Do I need to copy format?
+                for key in ["default", "secondaryFiles", "inputBinding", "loadContents"]:
                     if key in workflow_input:
                         updated_workflow_input[key] = workflow_input[key]
 
@@ -628,9 +644,17 @@ def fast_cwl_step_load(workflow, target_id, cwl_args=None, location=None):
 
                 step_in_source_with_step_id = step_in_source.replace("/", "_")  # to include both step name and id
 
+                # Check if it should be assumed optional (default field is present)
+                # NOTE: consider also checking if upstream step had scatter, so the
+                # output type should become array based on the scatter parameters
+                if "default" in step_in:
+                    upstream_step_output_type = ["null", upstream_step_output["type"]]
+                else:
+                    upstream_step_output_type = upstream_step_output["type"]
+
                 updated_workflow_input = {
                     "id": step_in_source_with_step_id,  
-                    "type": upstream_step_output["type"]
+                    "type": upstream_step_output_type
                 }
 
                 # No need to copy "secondaryFiles" for outputs from other steps
@@ -668,6 +692,21 @@ def fast_cwl_step_load(workflow, target_id, cwl_args=None, location=None):
             get_short_id(step_out, only_id=True)
         ))[0][1]
         step_out_with_step_id = step_out.replace("/", "_")  # to include both step name and id
+
+        # update output type in case of scatter
+        if "scatter" in selected_step:
+            selected_step_output = deepcopy(selected_step_output)                  # need to deepcopy, otherwise we change embedded tool's output
+            if isinstance(selected_step["scatter"], MutableSequence) \
+                and selected_step.get("scatterMethod") == "nested_crossproduct":
+                nesting = len(selected_step["scatter"])
+            else:
+                nesting = 1
+            for _ in range(0, nesting):
+                selected_step_output["type"] = {
+                    "type": "array",
+                    "items": selected_step_output["type"]
+                }
+
         workflow_outputs.append({
             "id": step_out_with_step_id,
             "type": selected_step_output["type"],
@@ -767,16 +806,16 @@ def get_short_id(long_id, only_step_name=None, only_id=None):
     If # is not present, use long_id. If only_step_name is True,
     return a short step name. If only_id is True, return a short
     id without step name. If part after symbol # includes three
-    sections separated by "/", discard the middle one. If part
-    after symbol # includes only one section separated by "/",
-    reset "only_step_name" and "only_id" to False as we don't
+    or more sections separated by "/", discard the middle ones.
+    If part after symbol # includes only one section separated by
+    "/", reset "only_step_name" and "only_id" to False as we don't
     know whether it was step name of "id"
     """
     fragment = urlsplit(long_id).fragment
     part = fragment if fragment != "" else long_id
 
-    if len(part.split("/")) == 3:                                  # if "id" has weird uid between step name and id
-        part = "/".join([part.split("/")[0], part.split("/")[2]])
+    if len(part.split("/")) > 2:                                  # if "id" has weird stuff between step name and id
+        part = "/".join([part.split("/")[0], part.split("/")[-1]])
 
     if len(part.split("/")) == 1:                                  # if fragment is only one word (based on "/")
         only_step_name, only_id = False, False
